package listFW.visitor;

import listFW.*;

/**
 * Computes the length of a list using anonymous inner class.
 * @author D. X. Nguyen
 * @since Copyright 2002 - DXN All rights reserved
 */
public class Length implements IListAlgo {

    public static final Length Singleton = new Length();

    private Length() {
    }

    /**
     * Returns 0 because the empty list has no element.
     * @param host an empty list.
     * @param nu not used.
     * @return Integer
     */
    public Object emptyCase(IEmptyList host, Object nu) {
        return new Integer(0);
    }


    /**
     * Passes the accumulated length (1) to the rest
     * and asks an annonymous inner class for help to
     * compute the length.
     * @param host a non-empty list.
     * @param nu not used.
     * @return Integer
     */
    public Object nonEmptyCase(INEList host, Object nu) {
        // call helper as an anonymous inner class:
        return host.getRest().execute(new IListAlgo() {

            public Object emptyCase(IEmptyList h, Object acc) {
                return acc;
            }

            public Object nonEmptyCase(INEList h, Object acc) {
                int accLen = 1 + ((Integer)acc).intValue();
                return h.getRest().execute(this, new Integer(accLen));
            }
        },  // end of anonymous inner class.
        new Integer(1));  // pass accumulated length of 1 to helper.
    }
}